/*!
  \file SkypeConnection.cpp

  \brief SkypeConnection implementation

  $Id: SkypeConnection.cpp,v 1.17 2006/04/17 06:44:11 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   03.08.2005 09:23:30 MSD
*/

#include "h/SkypeConnection.h"

#include <limits>

// VC 7 bug fix
#ifdef SKYPE_MSC
#undef max
#endif // SKYPE_MSC

Flush::Flush(const SimpleHandler_t& _sh, const ErrorHandler_t& _eh, bool _send_event)
	: sh(_sh), eh(_eh), send_event(_send_event)
{};

SkypeConnection::SkypeConnection(const std::string& _app_name, const SimpleHandler_t& _def, const LogFunc_t& _log_func)
		:  RawSkypeConnection<>(RawNotify_t(this, &SkypeConnection::Dispatcher), _app_name, _log_func),
		   default_cbk(_def), 
		   error_parser("^#[^ ]+\\s+ERROR\\s*(\\d*)\\s*(.*)$")
{}; 

unsigned SkypeConnection::RegisterCallback(const std::string& _regex, const SimpleHandler_t&  _f)
{
  FuncLst_t::iterator p = CreateRegExFuncPair(_regex);
  return p->second.simple_f.Register(_f);
};

unsigned SkypeConnection::RegisterCallback(const std::string& _regex, const RegExNotify_t& _f)
{
  FuncLst_t::iterator p = CreateRegExFuncPair(_regex);
  return std::numeric_limits<unsigned>::max() - p->second.regex_f.Register(_f);
};

void SkypeConnection::UnRegisterCallback(const std::string& _regex, const unsigned& _id)
{
  FuncLst_t::iterator p = regex_map.find(_regex);
  if(p == regex_map.end()) return;

  if(_id <= std::numeric_limits<unsigned short>::max()) {
	p->second.simple_f.UnRegister(_id);		  
  } else {	
	p->second.regex_f.UnRegister(std::numeric_limits<unsigned>::max() - _id);
  };
  
  if((p->second.simple_f.size() == 0) && (p->second.regex_f.size() == 0)) regex_map.erase(p);
};

SkypeConnection& SkypeConnection::operator<<(const Flush& _f)
{
  Assert(IsIdsEnabled());
  
  CmdFormatter_t& me = *this;
  me.flush();
  const int& id = LastNum();
  Assert(active_cmds.find(id) == active_cmds.end());
  active_cmds[id] = _f; 

  return *this;
};

void SkypeConnection::Dispatcher(const std::string& _msg)
{
  log(LlDbg, "%s: '%s' was received", __FUNCTION__, _msg.c_str());

  // Is _msg a response to a command?
  if(_msg.length()) if(_msg[0] == '#') {
	const std::string::size_type pos = _msg.find_first_of(' ');
	Assert(pos != std::string::npos);
	const int id = GetFromString<int>( _msg.substr(1, pos - 1));
	ActiveCmds_t::iterator p = active_cmds.find(id);
	Assert(p!=active_cmds.end());
	// Is response normal or error?
	boost::cmatch what;
	bool r = boost::regex_match(_msg.c_str(), what, error_parser);
	if(r) { // Error response was received
		Assert(what.size() == 3);
		const std::string& err_id_str = what[1].str();
		const int& err = err_id_str.length() ? GetFromString<int>(err_id_str) : 0;
		p->second.eh(err, what[2].str(), what[0].str());
	} else { // Normal response was received
		p->second.sh(_msg.substr(pos + 1, _msg.length()));
		if(p->second.send_event) SendEvent(_msg.substr(pos + 1, _msg.length()));
	};
	active_cmds.erase(p);
  } else { // _msg is notification
	SendEvent(_msg);
  };	
};

void SkypeConnection::SendEvent(const std::string& _event)
{
//  log(LlDbg, "%s: '%s' received", __FUNCTION__, _event.c_str());
  boost::cmatch what;
  
  // Try to find matched regexp
  bool found(false);
  for(FuncLst_t::iterator p=regex_map.begin();p!=regex_map.end();++p) {
//	log(LlDbg, "Checking regex: %s", p->second.regex.str().c_str());
	if(boost::regex_match(_event.c_str(), what, p->second.regex)) {
		found = true;
		RunCallBacks<SimpleHandler_t> simple_h(_event);
		p->second.simple_f.for_each(simple_h);
		
		RunCallBacks<RegExNotify_t> re_h(what);
		p->second.regex_f.for_each(re_h);
	};	
  };

  // If no regexp matches, call default handler
  if(!found) default_cbk(_event);
};	

SkypeConnection::FuncLst_t::iterator SkypeConnection::CreateRegExFuncPair(const std::string& _regex)
{
  FuncLst_t::iterator p = regex_map.find(_regex);
  if(p != regex_map.end()) return p;
  
  try {  
	const boost::regex rx(_regex); 
	RegExFuncPair_t re;
	re.regex = rx;
	std::pair<std::string, RegExFuncPair_t> pp(_regex, re);
  	return regex_map.insert(pp).first;
  } catch (boost::bad_expression& e) {
	throw AssertionViolation(e.what());
  };		
};

