/*!
  \file AttrsDesc.h

  \brief  Skype objects attribute description.  Base templates

  $Id: AttrsDesc.h,v 1.9 2006/05/27 13:00:48 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   16.08.2005 08:52:59 MSD
*/

#ifndef ATTRS_DESC_H
#define ATTRS_DESC_H

#include "SkypeExceptions.h"

#include <string>

#include <loki/AssocVector.h>
#include <loki/static_check.h>

namespace Ice {

/*!
  \brief This is a base class for constructing skype object attributes descriptions

  	 Every skype objects has different attributes.  Description of object's attribute is it's string name
	 suitable for GET/SET Skype API commands and several boolean options which describes the kind of attribute.
	
	 Classes which describes concrete object's attributes should be derived classes of this template.  
	 They should implement constructor which calls #Add method of this template class.

	 You can iterate through attribute descriptions stored by instance of this template via iterators.
	 You can obtain detailed attribute description by its name.  "Syntax sugar" operators [] are provided for most
	 frequently used values of attributes descriptions.
	 
  \param Attrs_ Object attribute type (usually some kind of enum type)
*/
template <typename Attrs> 
class AttrsDescriptionBase {
public:
	/*!
	  \brief Get attribute value associated with the string _idx
  
	  \param _idx Textual (string) representation of the object's attribute
	  
	  \throw SkypeInvalidAttr If there is no attribute value associated with the string _idx
	  
	  \return Attribute value associated with the string _idx
	*/
	const Attrs& operator[](const std::string& _idx) const;
	/*!
	  \brief  Get textual (string) representation of the attribute _idx

	  \param _idx Object's attribute 
	  
	  \return  Textual (string) representation of the object's attribute.  If there is no string representation
	           for attribute _idx (for example, #Add was never called for this attribute), empty string is returned.
	*/
	const std::string& operator[](const Attrs& _idx) const;
	/// Detailed attribute description
	struct Desc {
		/// Fast struct creation
		Desc(const std::string& _name, const bool& _can_get, const bool& _can_set, const bool _rep_at_obj_creation);
		/// Empty struct initialization
		Desc() {};
		/// Textual (string) representation of the object's attribute.
		std::string name;
		/// Can this attribute be GET (from Skype)?
		bool can_get;
		/// Can this attribute be SET?
		bool can_set;
		/*! 
		  \brief Is the attribute's value reported at Skype object creation (via notifications)?
		    
		    This flag is ignored if can_get is false.  It is used only with objects controlled
		    by managers, not standalone objects.
		*/   
		bool reported_at_obj_creation; 
	};
	/// Elements of this container
	typedef std::pair<Attrs, Desc> Elem_t;
	/// Constant iterator, points to Elem_t value type
	typedef typename Loki::AssocVector<Attrs, Desc>::const_iterator const_iterator;
	/// Points to the beginning of stored descriptions sequence
	const_iterator begin() const;
	/// Points to the "last + 1" element of stored descriptions sequence
	const_iterator end() const;
	/*!
	  \brief Get detailed description of the attribute

	  \param _idx Object's attribute
	
	  \throw SkypeInvalidAttr If there is no detailed description available of the attribute _idx

 	  \return Detailed description of attribute _idx 
	*/
	const struct Desc& GetDetails(const Attrs& _idx) const;
protected:
	/*!
	  \brief Add (or replace) attribute description

	  If there is an existing description of attribute _attrs, it is replaced by this new description.

	  \param _attr		Attribute
	  \param _attr_name	Textual (string) representation of the attribute _attr
	  \param _can_get	Can this attribute be GET (from Skype)?
	  \param _can_set	Can this attribute be SET?
	  \param _reported_at_obj_creation Is the attribute's value reported at Skype object creation (via notifications)? 
	  				   See documentation on #Ice::AttrsDescriptionBase::Desc also. 
	*/
	void Add(const Attrs& _attr, const std::string& _attr_name, const bool& _can_get = 1, const bool& _can_set = 0, 
		 const bool& _reported_at_obj_creation = 0);
	/// Default constructor. It is required by compiler. Does nothing.
	AttrsDescriptionBase() {};
private:
	/// Attribute <-> its detailed description
	Loki::AssocVector<Attrs, Desc> attrs_details;
	/// String representation of the attribute <-> attribute
	Loki::AssocVector<std::string, Attrs> name_to_attr;
	/// Empty string
	const std::string empty_str;
};

/// This template should be specialized for each Skype object's attribute type
/// Common instance of this template leads to compilation error 
template <typename Attrs>
class AttrsDescription : public AttrsDescriptionBase<Attrs> {
public:
	/*!
	  \brief Specializations should define their own constructors where #Ice::AttrsDescriptionBase<Attrs>::Add functions 
		 are called
	*/	 
	AttrsDescription() { LOKI_STATIC_CHECK(0, ATTRS_DESCRIPTION_FOR_THIS_TYPE_WAS_NOT_FOUND); };
};

/**********************************************************************************************************/
/************************************ Implementation ******************************************************/
/**********************************************************************************************************/

//************************************ AttrsDescriptionBase::Desc ****************************************//

template <typename Attrs> 
inline AttrsDescriptionBase<Attrs>::Desc::Desc(const std::string& _name, const bool& _can_get, 
					       const bool& _can_set, const bool _rep_at_obj_creation)
				   : name(_name), can_get(_can_get), can_set(_can_set), 
				   reported_at_obj_creation(_rep_at_obj_creation)
{};	
	
//************************************ AttrsDescriptionBase **********************************************//

template <typename Attrs> 
inline const Attrs& AttrsDescriptionBase<Attrs>::operator[](const std::string& _idx) const
{
  typename Loki::AssocVector<std::string, Attrs>::const_iterator p = name_to_attr.find(_idx);
  if(p == name_to_attr.end()) throw SkypeInvalidAttr(_idx);

  return p->second;
};

template <typename Attrs> 
inline const std::string& AttrsDescriptionBase<Attrs>::operator[](const Attrs& _idx) const
{
  const_iterator p = attrs_details.find(_idx);
  return (p == attrs_details.end()) ? empty_str : p->second.name;
};

template <typename Attrs> 
inline typename AttrsDescriptionBase<Attrs>::const_iterator
AttrsDescriptionBase<Attrs>::begin() const
{ return attrs_details.begin(); };

template <typename Attrs> 
inline typename AttrsDescriptionBase<Attrs>::const_iterator AttrsDescriptionBase<Attrs>::end() const
{ return attrs_details.end(); };

template <typename Attrs> 
inline const typename AttrsDescriptionBase<Attrs>::Desc&
AttrsDescriptionBase<Attrs>::GetDetails(const Attrs& _idx) const
{
  const_iterator p = attrs_details.find(_idx);
  if(p == attrs_details.end()) throw SkypeInvalidAttr("UNKNOWN", _idx);

  return p->second;
};	

template <typename Attrs> 
void AttrsDescriptionBase<Attrs>::Add(const Attrs& _attr, const std::string& _attr_name, const bool& _can_get,
				      const bool& _can_set, const bool& _reported_at_obj_creation)
{
  name_to_attr[_attr_name] = _attr;
  attrs_details[_attr] = Desc(_attr_name, _can_get, _can_set, _reported_at_obj_creation);
};	

}; // namespace Ice
#endif //ATTRS_DESC_H
