/*!
  \file Calls.h

  \brief Call-related stuff
  
  $Id: Calls.h,v 1.26 2006/10/22 10:49:14 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   01.08.2005 11:06:27 MSD
*/

#ifndef SK_CALLS_H
#define SK_CALLS_H

#include "SkypeObj.h"

namespace Ice {
/// Attributes of the Call
typedef enum { 
	clTimeStamp,
	clPartnerHandle,
	clPartnerDispname,
	clConfId,
	clType,
	clStatus,
	clFailureReason,
	clSubject,
	clPSTNStatus,
	clDuration,
	clPSTNNumber,
	clConfParticipantsCount,
	clConfParticipant,
	clVMDuration,
	clVMAllowedDuration,
	clSeen,
	clDTMF,
	clJoinConference,
	clVideoStatus,
	clVideoSendStatus,
        clVideoReceiveStatus	
} CallAttrs_t;

/// Description of Call object attributes
typedef AttrsDescription<CallAttrs_t> CallAttrsDesc_t;

/// Describes atribute of Skype object "Call"
template <>
class AttrsDescription<CallAttrs_t> : public AttrsDescriptionBase<CallAttrs_t> {
public:
	/// Fills description
	AttrsDescription();
};

// Forward declaration of Call object
class Call;

/// Watchers for the Call object
typedef StateWatchers<Call, CallAttrs_t> CallWatchers_t;

/*! 
  \brief This class represents call

  Never create objects of this class directly. CallManager is creator of these objects.
*/  
class Call : public SkypeObject<Call, CallAttrs_t, SkypeIdObjCmdFormatter> {
public:
    /*!
      \brief	Constructs Call object with DEFAULT error handler.  
  		  
	  	Default error handler doesn't handle any error at all.

      \param _sc	Reference to Skype Connection
      \param _id	Call identifier
      \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
    */
    Call(SkypeConnection& _sc, const int& _id, const LogFunc_t& _log_func = DummyLogger); 
     /*!
      \brief	Constructs Call object with NON-DEFAULT (user-supplied) error handler.  
  		  
      \param _sc	Reference to Skype Connection
      \param _id	Call identifier
      \param _eh	Error hanlder.  This functor will be called in case of errors.
      \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
    */
    Call(SkypeConnection& _sc, const int& _id, const ObjErrHandler_t& _eh, const LogFunc_t& _log_func = DummyLogger);
    /*!
      \brief Checks if object is alive or not. 

      \return alive if object is alive, finished if not alive and unknown if object doesn't held "status" attribute		
    */  
    LiveMode_t IsAlive() const;
private:
    /// Object-specific initialization.  Called from constructors 
    void Init();	
};


/// Possible types of calls search
typedef enum {
	srCalls,
	srActiveCalls,
	srMissedCalls
} CallSearchTypes_t;

/// Description of the different calls search types
typedef AttrsDescription<CallSearchTypes_t> CallSearchDesc_t;

/// Describes users search types
template <>
class AttrsDescription<CallSearchTypes_t> : public AttrsDescriptionBase<CallSearchTypes_t> {
public:
	/// Type of attributes of the oobjects returned by search procedure
	typedef CallAttrs_t ObjAttrs_t; 
	/// Type of objects returned by the search procedure
	typedef Call ObjType_t;
	/// Fills description
	AttrsDescription();
};


class CallManager : public SkypeObjManager<Call, CallAttrs_t, CallManager, CallSearchTypes_t, Searcher> {
public:
    /*!
      \brief	Constructs Call manager object with DEFAULT error handler.  
  		  
	  	Default error handler doesn't handle any error at all.

      \param _sc	Reference to Skype Connection
      \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
    */	
	CallManager(SkypeConnection& _sc, const LogFunc_t& _log_func = DummyLogger); 
     /*!
      \brief	Constructs Call manager object with NON-DEFAULT (user-supplied) error handler.  
  		  
      \param _sc	Reference to Skype Connection
      \param _eh	Error hanlder.  This functor will be called in case of errors.
      \param _log_func	Reference to the logger. This logger will be used to log any messages inside the object.
    */	
	CallManager(SkypeConnection& _sc, const Call::ObjErrHandler_t& _eh, const LogFunc_t& _log_func = DummyLogger); 
	/*!
	  \brief Parses event related to calls
  
	  \param _match Parsed regular expression
	  \return Call identifier
	*/
	int ParseEvent(const boost::cmatch& _match);
	/*!
  	 \brief	Initiates a call to remote subscriber(s).
  
  	 \param _name Remote subscriber(s) name(s) (skype name(s)). If the number of subscribers
		      more than 1, skype names should be delimeted by comma (',')

	*/
	void MakeCall(const std::string& _name);
private:
	/// Connection to Skype
	SkypeConnection& sc;
	void Init();
};

}; // namespace Ice
#endif // SK_CALLS_H
