/*!
  \file SkypeExceptions.h

  \brief ++Skype library exceptions

  $Id: SkypeExceptions.h,v 1.9 2006/06/12 14:34:53 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   19.07.2005 16:12:12 MSD
*/

#ifndef SKYPE_EXCEPTIONS_H
#define SKYPE_EXCEPTIONS_H

#include <stdexcept>
#include <string>

/// All skype run-time errors can be catched via references to this base class 
class SkypeError : public std::runtime_error {
public:
	/// Fills buffer for what() method
	SkypeError(const std::string& _arg) : std::runtime_error(_arg) {};  	
};

/// Application was disconnected from skype for some reason 
class SkypeDisconnect : public SkypeError {
public:
	/// Fills buffer for what() method
	SkypeDisconnect(const std::string& _arg) : SkypeError(_arg) {};  	
};

/// User didn't allow application to connect to skype
class SkypeAccessDenied : public SkypeError {
public:
	/// Fills buffer for what() method
	SkypeAccessDenied() : SkypeError("Skype user doesn't allow this application to connect to Skype") {};  	
};

/// User has requested skype object's attribute which doesn't have the description
class SkypeInvalidAttr : public SkypeError {
public:
	/// Fills buffer for what() method
	SkypeInvalidAttr(const std::string& _arg, const int _attr = -1)
		: SkypeError("Invalid atrribute name or no data for this atribute: " + _arg), attr(_attr) {};
	/// Returns integer representation of the attribute
	const int& GetAttr() const { return attr; };
private:
	int attr;
};

/// Error was reported by Skype in response to application command
class SkypeAPIError : public SkypeError {
public:
	/*!
	  \brief Constructs SkypeAPIError object 
  
	  \param _obj_name Name of the Skype object where exception was thrown
	  \param _err_code Skype error code	
	  \param _err_txt  Textual error representation
	  \param _err_msg  Unmodified error message received from Skype
	*/
	SkypeAPIError(const std::string& _obj_name, const int& _err_code, 
								const std::string& _err_txt, const std::string& _err_msg);
	/// Returns Skype error code	
	const int& GetErrNo() const;
	/// Returns textual error representation
	const std::string& GetErrTxt() const;
	/// Returns the name of the Skype object where exception was thrown
	const std::string& GetObjName() const;
	/// Returns unmodified error message received from Skype
	const std::string& GetErrMsg() const;
	/// Returns human-readable error description
	virtual const char* what() const throw();
	/// Virtual destructor with throw-spec is required for correct compilation
	virtual ~SkypeAPIError() throw() {};
private:
	/// Name of the Skype object where exception was thrown
	const std::string obj_name;
	/// Skype error code
	const int err_code;
	/// Textual error representation
	const std::string err_txt;
	/// Unmodified error message received from Skype
	const std::string err_msg;
	/// String for #what() method
	std::string what_str;
};

/*! 
   \brief This exception is thrown by the Search method of skype object managers if Skype returned error message
   	  in response to the search query

   	  The common reason of this error is invalid value of the search parameter
*/
class SkypeInvalidSearch : public SkypeAPIError {
public:
	/*!
	  \brief Constructs SkypeInvalidSearch object 
  
	  \param _obj_name Name of the Skype object where exception was thrown
	  \param _err_code Skype error code	
	  \param _err_txt  Textual error representation
	  \param _err_msg  Unmodified error message received from Skype
	*/
	SkypeInvalidSearch(const std::string& _obj_name, const int& _err_code, 
								const std::string& _err_txt, const std::string& _err_msg);
};

/// Used for run-time assertions
class AssertionViolation : public std::logic_error {
public:
	/*!
	  \brief Constructs AssertionViolation object

	  \param _arg Human-readable description of assertion violation
	*/  
	AssertionViolation(const std::string& _arg) : std::logic_error(_arg) {};  	
};

#endif // SKYPE_EXCEPTIONS_H

