/*!
  \file utils.h

  \brief  Some useful library routines
  
  $Id: utils.h,v 1.20 2006/05/27 13:00:48 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   20.07.2005 09:28:05 MSD
*/

#ifndef SKYPE_UTILS_H
#define SKYPE_UTILS_H

#include "Cfg.h"
#include "SkypeExceptions.h"

extern "C" {
#include <stdlib.h>
#include <math.h>
};

#include <loki/static_check.h>
#include <loki/Typelist.h>
#include <loki/TypeTraits.h>
#include <loki/Functor.h>

#include <boost/date_time/posix_time/posix_time.hpp>

#include <map>
#include <list>

/// Run-time assertions.  Should do nothing in release build
template <DbgMode_t DBG> inline void AssertFunc(const bool tst, const char* msg) {}; 

/*!
   \brief Run-time assertion.  Debug-time specialization.  

   \param tst 	Test expression
   \param msg 	Error message sent with exception in case if tst is false
   
   \throws AssertionViolation exception is thrown if tst is false.  
*/   
template <> inline void AssertFunc<debug>(const bool tst, const char* msg)
{
  if(!tst) throw AssertionViolation(msg);	  			
};

#define FILELINE3(f, l) f ":" #l
#define FILELINE2(f, l) FILELINE3(f, l)
#define FILELINE FILELINE2(__FILE__, __LINE__) 

/// User friendly macros to test assertions
#define Assert(tst) AssertFunc<DbgMode>(tst, "AssertionViolation was thrown at " FILELINE);

/*!
  \brief Converts string to C type

  \param value string
  
  \throw std::range_error if value can not be converted
  
  \return Converted value
*/  
template <class C> C GetFromString(const std::string& value);

/*!
  \brief Generic numerator.  Designed for use with numeral data types.

  \param T The type of numerator counter
*/
template <class T> class Numerator {
public:
	/*!
	   \brief Numerator initialization

	   \param _start Initial numerator value
	*/   
	Numerator(T _start = 0) : id(_start) {};
	/// Return last numerator value
	T LastNum() const { return id; };
	/// Increments numerator and returns its new value
	T IncrNum() { return ++id; };
private:
	/// counter
	T id;
};

/*!
  \brief Object's registrator

  This object is used to register/unregister objects, iterate through objects.
  
  \param Obj Type of registrated objects
  \param ID  Type of objects identifiers	
*/  
template <class Obj, typename ID = unsigned> class Registrator {
public:
	/// Initialization
	Registrator();
	/*!
	  \brief Register object
  
	  	  Registrates and stores the object inside container.
		  See documentation on #for_each for details about calling this method inside its functor argument.
 
	  \param _obj	Registered object
	  
	  \return Identifier of registered object
	*/
	ID Register(const Obj& _obj);
	/*!
	  \brief Unregister object.
  
	  	 See documentation on #for_each for details about calling this method inside its functor argument.

	  \param _id  Identifier of previously registered object

	  \return true if object was unregistered, false if there was no registered object with identifier equal to _id
	*/
	bool UnRegister(const ID& _id);
	/// Unregisters all previously registered objects. Use with caution!
	void clear();
	/// This type is used for registered objects counting 
	typedef typename std::list<ID>::size_type size_type;
	/*!
	  \brief Counts the quantity of registered objects

	  \return Quantity of registered objects 
	*/
	size_type size() const;
	/// Type for storing registered objects
	typedef typename std::map<ID, Obj> ObjMap_t;
	/// Function type suitable for use with #for_each method.  First argument is object identifier,  
	/// Second argument is the object itself. See documentation on #for_each for additional details.
	typedef Loki::Functor<void, LOKI_TYPELIST_2(const ID&, Obj&)> ObjHandler_t;
	/*!
	  \brief Calls functor _func for each registered object
  
	  	 If _func calls #UnRegister method of this class, it is executed only AFTER all currently registered objects 
		 are handled (_func is executed with all currently /before entering for_each/ registered objects).

		 If _func calls #Register method of this class, it is executed only AFTER all currently registered objects 
		 are handled (_func is executed with all currently /before entering for_each/ registered objects).

	  \param _func Functor to be called.
	*/
	void for_each(const ObjHandler_t& _func);
	/// Returns constant reference to the collection of registered objects
	const ObjMap_t& GetObjects() const;
private:	
	/// Registered objects
	ObjMap_t objects;
	/// Objects, registration for which was requested inside #for_each functor argument
	ObjMap_t req_for_reg_objs;
	/// Object's numerator
	Numerator<ID> numerator;
	/// for_each method is active now
	bool for_each_is_called;
	/// RAII for for_each method. We have to use RAII because of exception-safety of for_each method
	class ForEachLocker {
	public:
		/// Sets for_each_is_called flag of host class to true
		ForEachLocker(Registrator<Obj, ID>& _host);
		/// Sets for_each_is_called flag of host class to false 
		~ForEachLocker();
	private:
		/// Reference to host class
		Registrator<Obj, ID>& host;
	};
	// ForEachLocker is needed the access to for_each_is_called member	
	friend class ForEachLocker;
	/// Identifiers of objects for which #UnRegister was called inside #for_each functor argument
	std::list<ID> unregistered_ids;
};

namespace Impl {
	
/*!
  \brief  Sleep for specified amount of time

  \param _td Sleep duration
*/
void Sleep(boost::posix_time::time_duration _td);

};


/**********************************************************************************************************/
/************************************ Implementation ******************************************************/
/**********************************************************************************************************/
namespace Impl {
  /// enum Impl::GetFromStringAlgoSelector 	
  /// Converter selector
  enum GetFromStringAlgoSelector { Unknown, Strings, Ints, UnsignedInts, Float, Bool };

  /// Converts string to unknown type C.  Compile-time error occurs.
  template <typename C>
  C GetFromStringImpl(const std::string& value, Loki::Int2Type<Unknown>)
  {
    LOKI_STATIC_CHECK(0, Unknown_Type_To_Get_From_String); 	
  };	

  /// Converts string to one of signed integer types
  template <typename C>
  C GetFromStringImpl(const std::string& value, Loki::Int2Type<Strings>)
  {
    return value;
  };	

    /// Converts string to one of signed integer types
  template <typename C>
  C GetFromStringImpl(const std::string& value, Loki::Int2Type<Ints>)
  {
    char* endptr = NULL; 	   
    long int r = strtol(value.c_str(), &endptr, 0);
    if(endptr != NULL) if(*endptr != '\0') throw std::range_error(value);
    if((r==LONG_MAX)||(r==LONG_MIN)) throw std::range_error(value);
    
    return r;
  };	

  /// Converts string to one of unsigned integer types
  template <typename C>
  C GetFromStringImpl(const std::string& value, Loki::Int2Type<UnsignedInts>)
  {
    char* endptr = NULL; 	   
    unsigned long r = strtoul(value.c_str(), &endptr, 0);
    if(endptr != NULL) if(*endptr != '\0') throw std::range_error(value);
    if(r==ULONG_MAX) throw std::range_error(value);
    
    return r;
  };	

  /// Converts string to one of float types
  template <typename C>
  C GetFromStringImpl(const std::string& value, Loki::Int2Type<Float>) throw(std::range_error)
  {
    char* endptr = NULL; 	   
    double r = strtod(value.c_str(), &endptr);
    if(endptr != NULL) if(*endptr != '\0') throw std::range_error(value);
    
    if(r == HUGE_VAL) throw std::range_error(value);
    
    return r;
  };

  /// Converts string to boolean 
  template <typename C>
  bool GetFromStringImpl(const std::string& value, Loki::Int2Type<Bool>)
  {
	if((value == "true")||(value == "1")||(value == "TRUE")||(value == "ON")||(value == "on")) return true;
	if((value == "false")||(value == "0")||(value == "FALSE")||(value == "OFF")||(value == "off")) return false;
  	
	throw std::range_error(value);
  };	  
};

template <class C> C GetFromString(const std::string& value)
{
   enum { algo = 
	   Loki::TypeTraits<C>::isStdSignedInt ? Impl::Ints : 
		   ( Loki::TypeTraits<C>::isStdUnsignedInt ? Impl::UnsignedInts :
				( Loki::TypeTraits<C>::isStdFloat ? Impl::Float : 
				( Loki::IsSameType<bool, C>::value ? Impl::Bool : 
				( Loki::IsSameType<std::string, C>::value ? Impl::Strings : Impl::Unknown )))
		   )
  };
  
  return Impl::GetFromStringImpl<C>(value, Loki::Int2Type<algo>());
};

//************************************ Registrator::ForEachLocker ****************************************//

template <class Obj, typename ID>
inline Registrator<Obj, ID>::ForEachLocker::ForEachLocker(Registrator<Obj, ID>& _host)
		 : host(_host)
{
  host.for_each_is_called = true;	
};	

template <class Obj, typename ID>
inline Registrator<Obj, ID>::ForEachLocker::~ForEachLocker()
{
  host.for_each_is_called = false;	
};	

//************************************ Registrator *******************************************************//

template <class Obj, typename ID>
Registrator<Obj, ID>::Registrator()
		: for_each_is_called(false)
{};

template <class Obj, typename ID>
ID Registrator<Obj, ID>::Register(const Obj& _obj)
{
  const ID& id = numerator.IncrNum();
  ObjMap_t& objs = (for_each_is_called) ? req_for_reg_objs : objects;
  
  objs[id] = _obj;
  
  return id;
};	

template <class Obj, typename ID>
bool Registrator<Obj, ID>::UnRegister(const ID& _id) 
{
  if(for_each_is_called) {
	unregistered_ids.push_back(_id);
  	return false;
  };
  
  typename ObjMap_t::iterator p = objects.find(_id);
  bool r = (p != objects.end());
  if(r) objects.erase(p);

  return r;
};

template <class Obj, typename ID>
void Registrator<Obj, ID>::for_each(const ObjHandler_t& _func)
{
  // you shoudn't call for_each method inside _func!	
  Assert(!for_each_is_called);
  Assert(unregistered_ids.empty()); 
  Assert(req_for_reg_objs.empty());
  
  typedef typename ObjMap_t::iterator IT;
  {
    ForEachLocker locker(*this);
    ObjHandler_t func(_func);
    for(IT p = objects.begin();p != objects.end();++p) func(p->first, p->second);
  };
  
  while(!unregistered_ids.empty()) {
	UnRegister(unregistered_ids.front());
	unregistered_ids.pop_front();
  };
  
  for(IT p = req_for_reg_objs.begin();p != req_for_reg_objs.end();++p) {
	objects.insert(*p);
  };
  req_for_reg_objs.clear();
};	

template <class Obj, typename ID>
inline void Registrator<Obj, ID>::clear()
{ objects.clear(); };

template <class Obj, typename ID>
inline typename Registrator<Obj, ID>::size_type Registrator<Obj, ID>::size() const
{ return objects.size(); };


template <class Obj, typename ID>
inline const typename Registrator<Obj, ID>::ObjMap_t& Registrator<Obj, ID>::GetObjects() const
{ return objects; };

#endif // SKYPE_UTILS_H
