/*!
  \file Loggers.cpp

  \brief Tests of different loggers included in ++Skype
  
  $Id: Loggers.cpp,v 1.3 2006/01/17 09:45:18 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   26.07.2005 10:16:02 MSD
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/output_test_stream.hpp>

#include <sstream>

#include <h/Loggers.h>

using namespace boost::unit_test;
using boost::test_tools::output_test_stream;

/// Testing different loggers constructors
void LoggersConstruction() 
{
  // Try to open file in "/" directory - should fail	
  FileLogger<> l_name1("NO-DIRECTORY/++Skype-test-should-not-be-opened");
  std::string err_msg;
  BOOST_CHECK(!l_name1.IsReady(err_msg));
  BOOST_CHECK(err_msg.length());
  err_msg.clear();
  
  // Try to open file in local directory
  FileLogger<> l_name2("./LoggersTest.log");
  BOOST_CHECK(l_name2.IsReady(err_msg));
  BOOST_CHECK(err_msg.empty()); // Nothing should change in err_msg

  // Try to open file stdout
  FileLogger<> l_file(stdout);
  BOOST_CHECK(l_name2.IsReady(err_msg));
  BOOST_CHECK(err_msg.empty()); // Nothing should change in err_msg

  // Try to open stream logger (stderr)
  StreamLogger<> l_stream1;
  BOOST_CHECK(l_stream1.IsReady(err_msg));
  BOOST_CHECK(err_msg.empty()); // Nothing should change in err_msg
  
  // Try to open stream logger (ostream_test_stream)
  output_test_stream out;
  StreamLogger<> l_stream2;
  BOOST_CHECK(l_stream2.IsReady(err_msg));
  BOOST_CHECK(err_msg.empty()); // Nothing should change in err_msg
};

template <class T> 
void TstOut(T& l, LogLevel_t _lvl, const char* _fmt, ...)
{
  const char* n = __FUNCTION__;	
  va_list args;	
  va_start(args, _fmt);
  l(_lvl, _fmt, args);
  va_end(args);	
};

void LoggersOutputTest()
{
  output_test_stream out_stream;
  StreamLogger<> l(out_stream);
  std::string str;
  BOOST_CHECK(l.IsReady(str));
  BOOST_CHECK(str.empty()); // Nothing should change in str
  TstOut(l, LlInfo, "Test: '%d' '%s' '%.2f'", 12, "string", 12.25); 
  char test_check[] = "[Info] Test: '12' 'string' '12.25'\n";
  BOOST_CHECK(out_stream.is_equal(test_check));
  l.SetMinLogLvl(LlInfo);
  TstOut(l, LlDbg, "Should not appear in log because of LlDbg < LlInfo");
  BOOST_CHECK(out_stream.is_empty());
  
  
  // Visual part of the test
  FileLogger<LvlTimeLogPrefix> l0(stderr);
  str.clear();
  BOOST_CHECK(l0.IsReady(str));
  BOOST_CHECK(str.empty()); // Nothing should change in str
  TstOut(l0, LlInfo, "Test: '%d' '%s' '%.2f'", 12, "string", 12.25); 

  StreamLogger<> l1;
  BOOST_CHECK(l1.IsReady(str));
  BOOST_CHECK(str.empty()); // Nothing should change in str 
  size_t sz = strlen(test_check);
  test_check[sz-1] = '\0';
  std::ostringstream s;
  s << "Now you should see the string \"" << test_check << "\" at stderr";
  BOOST_MESSAGE(s.str());
  TstOut(l1, LlInfo, "Test: '%d' '%s' '%.2f'", 12, "string", 12.25); 

  #if defined(__unix__) && (!defined(__CYGWIN__))
  SysLogger* slp = new SysLogger("++SKYPE test", LOG_PID, LOG_LOCAL7);
  SysLogger& sl = *slp;
  BOOST_CHECK(sl.IsReady(str));
  BOOST_CHECK(str.empty()); // Nothing should change in str
  std::ostringstream s1;
  s1 << "Now you should see the string \"" << test_check << "\" at syslog output LOG_LOCAL7, LOG_INFO";
  BOOST_MESSAGE(s1.str());
  TstOut(sl, LlInfo, "Test: '%d' '%s' '%.2f'", 12, "string", 12.25); 
  #endif 
};

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
    test_suite* test= BOOST_TEST_SUITE("Loggers test");
    test->add(BOOST_TEST_CASE(&LoggersConstruction));
    test->add(BOOST_TEST_CASE(&LoggersOutputTest));
    
    return test;
}







