/*!
  \file Privileges.cpp

  \brief Check user privileges 

  $Id: Privileges.cpp,v 1.10 2006/06/14 13:31:54 tarle Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   09.08.2005 10:33:49 MSD
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_suite.hpp>

#include <algorithm>
#include <typeinfo>
#include <list>

#include "h/Privileges.h"
#include "h/utils.h"
#include "h/Cfg.h"

#include "SkypeTest.h"

using namespace boost::unit_test;
using namespace Ice;

SkypeConnection* SC;

void DefaultCallback(const std::string& s)
{
  fprintf(stdout, "%s: '%s'\n", __FUNCTION__, s.c_str());	
};

template <typename Attrs>
void PrintValue(const std::pair<Attrs, std::string>& _msg)
{
  fprintf(stdout, "%s/%s: Atribute %d value is '%s'\n", __FUNCTION__, typeid(_msg.first).name(), _msg.first, _msg.second.c_str());	
};

template<class Test, class TestAttr>
class CallBack {
public:
	void Do(Test&, const TestAttr& _tst_attr, const std::string& _tst_v)
	{ tst_v = _tst_v;  tst_attr = _tst_attr; };	
	std::string tst_v;
	TestAttr tst_attr;
};
 
template<class Test, class TestAttr>
void WatchTest(Test& test, TestAttr attr)
{
  typedef StateWatchers<Test, TestAttr> Watchers_t;
  Watchers_t w;
  typename Watchers_t::AttrSet_t attr_set;
  CallBack<Test, TestAttr> test_cbk;
  typename Watchers_t::ObjChanged_t cbk(&test_cbk, &CallBack<Test, TestAttr>::Do);
//  typename Watchers_t::WatcherDetailsPtr_t wd1(new typename Watchers_t::WatcherDetails(attr_set, cbk));  
  unsigned ids[5];

  for(int i=0;i<5;i++) ids[i] = w.RegisterWatcher(attr_set, cbk);
  w(test, attr, "test");
  BOOST_CHECK(test_cbk.tst_v.length() == 0);
  
  for(int i=0;i<2;i++) w.UnRegisterWatcher(ids[i]);
  BOOST_CHECK_NO_THROW(w.UnRegisterWatcher(ids[0]));
  BOOST_CHECK_NO_THROW(w.clear());
  
//  typename Watchers_t::WatcherDetailsPtr_t wd2(new typename Watchers_t::WatcherDetails(attr_set, cbk, true));  
  w.RegisterWatcher(attr_set, cbk, true);
  w(test, attr, "test");
  BOOST_CHECK(test_cbk.tst_v == "test");
  BOOST_CHECK(test_cbk.tst_attr == attr);
};

void WatchTestPriv()
{
  Privileges priv(*SC);	
  WatchTest<Privileges, PrivAttrs_t>(priv, plSkypeIn);	
};

template<class Test, class TestAttr, class Result> 
void DoTest(const std::list<TestAttr>& _attrs)
{
  SkypeConnection& sc = *SC;
  Result r;
  const boost::posix_time::seconds tm(1);
 
  sc << DisableIds << "PROTOCOL 99" << flush << EnableIds;
  ProcessSkypeMsgs(sc, tm);
  
  Test test(sc, sc.GetLog());
  
  const TestAttr& tst_attr = *_attrs.begin();
  if(DbgMode == debug) {
	BOOST_CHECK_THROW(test[tst_attr], AssertionViolation);
  };
  
  typename Test::const_iterator p = test.find(tst_attr);
  BOOST_CHECK(p == test.end());
  
  test.AskAttr(tst_attr);
  ProcessSkypeMsgs(sc, tm);
  BOOST_CHECK_NO_THROW(r = GetFromString<Result>(test[tst_attr]));
  p = test.find(tst_attr);
  BOOST_CHECK(p == test.begin());

  typename std::list<TestAttr>::const_iterator i = _attrs.begin(); 
  for(++i;i != _attrs.end(); ++i) {
	test.AskAttr(*i);
  };
  ProcessSkypeMsgs(sc, tm); 
  std::for_each(test.begin(), test.end(), PrintValue<TestAttr>);
};

void PrivilegesTest()
{
  SkypeConnection& sc = *SC;
  
  std::list<PrivAttrs_t> attrs;
  attrs.push_back(plSkypeOut);
  attrs.push_back(plSkypeIn);
  attrs.push_back(plVoiceMail);
  
  DoTest<Privileges, PrivAttrs_t, bool>(attrs);
};

void ProfileTest()
{
  SkypeConnection& sc = *SC;

  std::list<ProfileAttrs_t> attrs;
  attrs.push_back(prPSTNBalance);
  attrs.push_back(prPSTNBalanceCurr);
  attrs.push_back(prFullName);
  attrs.push_back(prBirthday);
  attrs.push_back(prSex);
  attrs.push_back(prLanguages);
  attrs.push_back(prCountry);
  attrs.push_back(prProvince);
  attrs.push_back(prCity);
  attrs.push_back(prPhoneHome);
  attrs.push_back(prPhoneOffice);
  attrs.push_back(prPhoneMobile);
  attrs.push_back(prHomePage);
  attrs.push_back(prAbout);
  attrs.push_back(prMoodText);
  attrs.push_back(prTimeZone);
  attrs.push_back(prCallNoAnswerTimeout);
  attrs.push_back(prCallForwardRules);  
  
  DoTest<Profile, ProfileAttrs_t, unsigned>(attrs);
};


test_suite* init_unit_test_suite(int argc, char* argv[])
{
    test_suite* test= BOOST_TEST_SUITE("Privileges and Profile unit test");
    const boost::posix_time::seconds tm(1);

    static FileLogger<> fl(stderr, LlDbg);
    LogFunc_t log(&fl, &FileLogger<>::operator());
    static SimpleHandler_t def(DefaultCallback);
    static SkypeConnection scl(argv[0], def, log);
    scl.Attach();
   
    SC = &scl;
    ProcessSkypeMsgs(scl, tm);
   
    test->add(BOOST_TEST_CASE(&WatchTestPriv));
    test->add(BOOST_TEST_CASE(&PrivilegesTest));
    test->add(BOOST_TEST_CASE(&ProfileTest));
    return test;
};

