/*!
  \file unix/LowLevel-X11.cpp

  \brief Low-level ++Skype API.  Unix implementation. X11 variant

  $Id $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   18.10.2006 23:29:01 MSD
*/

#include "../h/Cfg.h"

#ifdef USE_X11_INTERFACE
#ifdef SKYPE_NIX

#include "../h/LowLevel.h"

namespace Impl {

  Bool IsMyXEvent(Display *display, XEvent *event, XPointer arg)
  { 
     UnixX11RawSkypeConnection_t* sk = reinterpret_cast<UnixX11RawSkypeConnection_t*>(arg);
     return sk->IsMyXEvent(*event);
  };

}; // namespace Impl

UnixX11RawSkypeConnection_t::RawSkypeConnection(const RawNotify_t& _cbk, const std::string& _app_name, LogFunc_t _log_func) 
			: SkypeCmdFormatter< RawSkypeConnection<os_nix> >(*this),
			  LogAbility(_log_func),
			  cbk(_cbk), app_name(_app_name), attach_result(false), 
			  display(0), skype_win(None), my_win(None), 
			  a2(None), a1(None)  
{};	

void UnixX11RawSkypeConnection_t::Attach(Display* _display)
{
  if(attach_result) return;

  should_close_display = false;

  display = _display;
  my_win = XCreateSimpleWindow(display, DefaultRootWindow(display), 1, 1, 1, 1, 1, 1, 1);
  XSelectInput(display, my_win, ClientMessage);

  a2 = XInternAtom(display, "SKYPECONTROLAPI_MESSAGE", false);
  if(a2 == None) throw SkypeDisconnect("Can't find Skype-related X11 atom SKYPECONTROLAPI_MESSAGE");
  a1 = XInternAtom(display, "SKYPECONTROLAPI_MESSAGE_BEGIN", false);
  if(a1 == None) throw SkypeDisconnect("Can't find Skype-related X11 atom SKYPECONTROLAPI_MESSAGE_BEGIN"); 

  Atom skype_inst = XInternAtom(display, "_SKYPE_INSTANCE", True);
  if(skype_inst == None) throw SkypeDisconnect("Can't find Skype-related X11 atom _SKYPE_INSTANCE");
  
  Atom type_ret;
  int format_ret;
  unsigned long nitems_ret;
  unsigned long bytes_after_ret;
  unsigned char *prop;
  int status;

  status = XGetWindowProperty(display, DefaultRootWindow(display), skype_inst, 0, 1, False, XA_WINDOW, &type_ret, &format_ret, &nitems_ret, 
			      &bytes_after_ret, &prop);

  // sanity check
  if(status != Success || format_ret != 32 || nitems_ret != 1) throw SkypeDisconnect("Couldn't find Skype instance");
  else skype_win = * (const unsigned long *) prop & 0xffffffff;

  std::string cmd = "NAME ";
  cmd.append(app_name);

  flush(cmd);

  // Waiting for "OK" response from the Skype.  All other responses are interpreted as Access denied
  while(cached_replies.begin() == cached_replies.end()) ProcessXEvents();
 
  if(cached_replies[0].compare("OK")) throw SkypeAccessDenied();

  cached_replies.erase(cached_replies.begin()); 

  attach_result = true;
};

void UnixX11RawSkypeConnection_t::Attach(const std::string& _display)
{
  if(attach_result) return;

  Display* disp;
  const char* s = (_display.length()) ? _display.c_str() : NULL;
  
  disp = XOpenDisplay(s);
  if(disp == NULL) throw SkypeDisconnect("Can't open connection to X-Server");
  
  Attach(disp);
  
  should_close_display = true;
};

void UnixX11RawSkypeConnection_t::Detach()
{
  if(attach_result) {
	XDestroyWindow(display, my_win);
	if(should_close_display) XCloseDisplay(display);
	attach_result = false;
  };	
};

void UnixX11RawSkypeConnection_t::flush(const std::string& _cmd)
{
  log(LlDbg, "%s: Sending to SKYPE: '%s'", __PRETTY_FUNCTION__, _cmd.c_str());	

  unsigned int pos = 0;
  
  XEvent e;

  e.xclient.type = ClientMessage;
  e.xclient.message_type = a1; // leading message
  e.xclient.display = display;
  e.xclient.window = my_win;
  e.xclient.format = 8;

  do {
	unsigned int i;
	for(i=0;i< 20 && i + pos <= _cmd.length();++i) e.xclient.data.b[i] = _cmd[i+pos];
	XSendEvent(display, skype_win, False, 0, &e);
	e.xclient.message_type = a2; // following messages
	pos += i;
  } while(pos <= _cmd.length());

  XSync(display, False);
};

void UnixX11RawSkypeConnection_t::flush()
{
  SkypeCmdFormatter<UnixX11RawSkypeConnection_t>::flush();
}

bool UnixX11RawSkypeConnection_t::operator()()
{
  Assert(attach_result);

  bool res = false;

  while(cached_replies.begin() != cached_replies.end()) {
	cbk(cached_replies[0]);
	cached_replies.erase(cached_replies.begin());
	res = true;
  };

  ProcessXEvents();

  return res;
};

bool UnixX11RawSkypeConnection_t::IsMyXEvent(const XEvent& _ev) const 
{
  bool r =  (_ev.xclient.format == 8) && (_ev.type == ClientMessage);
  r = r && (_ev.xclient.message_type == a1 || _ev.xclient.message_type == a2);

  return r;
};

void UnixX11RawSkypeConnection_t::ProcessXEvents()
{
  XEvent ev;

  Bool r;
  do {
	r = XCheckIfEvent(display, &ev, Impl::IsMyXEvent, reinterpret_cast<char*>(this));  
	if(!r) break;
	
	char buf[ 21 ]; // can't be longer
	int i;
	for(i = 0; i < 20 && ev.xclient.data.b[i] != '\0';++i) buf[i] = ev.xclient.data.b[i];
	buf[i] = '\0';

	if(i<20) { // end of the msg
	   cur_reply += buf;
	   cached_replies.push_back(cur_reply);
	   cur_reply.clear();
	} else if(ev.xclient.message_type == a1) { // new message has been received
	   if(cur_reply.length()) cached_replies.push_back(cur_reply);
	   cur_reply = buf;
	} else cur_reply += buf; // middle part of the message	
  } while(r);
};

UnixX11RawSkypeConnection_t::~RawSkypeConnection()
{
  if(attach_result) {	
	XDestroyWindow(display, my_win);
	if(should_close_display) XCloseDisplay(display);
  };	
};

#endif // USE_X11_INTERFACE
#endif // SKYPE_NIX 

